-- #########################################################################
-- #                                                                       #
-- # License GPLv3: https://www.gnu.org/licenses/gpl-3.0.html              #
-- #                                                                       #
-- # This program is free software; you can redistribute it and/or modify  #
-- # it under the terms of the GNU General Public License version 2 as     #
-- # published by the Free Software Foundation.                            #
-- #                                                                       #
-- # This program is distributed in the hope that it will be useful        #
-- # but WITHOUT ANY WARRANTY; without even the implied warranty of        #
-- # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
-- # GNU General Public License for more details.                          #
-- #                                                                       #
-- #########################################################################
--
-- =============================================================================
-- ETHOS artificial horizon draw library
-- File:   : drawLib.lua
-- Author  : RNDr.Vladimir Pribyl, CSc. (VPRHELI)
-- History : Date        Version Author   Comment
--           ----------  ------- -------- ------------------------------------
--           16.02.2025  0.0.1   VPRHELI  initial version
-- =============================================================================

local drawLib     = {}
local conf        = nil
local libs        = nil

local frameX = 50   -- X frame size for pitch attitude markers
local frameY = 18   -- Y frame size for pitch attitude markers

-- widget variables based on zone size
local x, y
local w, h
local r
local scalePitch
local lineOffset
local lineCount
local centerX, centerY
local minY, maxY
local minX, maxX
local yy
local winglen
local STD_w, STD_h
local XL_w,  XL_h
local XXL_w, XXL_h
  
-- #################################################################### 
-- # drawLib.init                                                     #
-- ####################################################################
function drawLib.init(param_conf, param_libs)
  --print ("### drawLib.init ()")
  conf   = param_conf 
  libs   = param_libs
  
  return drawLib
end
-- #################################################################### 
-- #  getSourceValue                                                  # 
-- #################################################################### 
function drawLib.getSourceValue(input)
  if input == nil then
    return 0
  end
  local value = 0  
  if input:state() then
    value = input:value()
  end
  return value
end
-- #################################################################### 
-- #  changeDrawVar                                                   # 
-- #################################################################### 
function drawLib.changeDrawVar (widget)
  --print("#### changeDrawVar()")
  x           = 0
  y           = 0
  w           = widget.zoneWidth
  h           = widget.zoneHeight
  scalePitch  = math.floor(widget.zoneHeight / 60 + 0.5)
  lineOffset  = math.floor(widget.zoneHeight / 60 + 0.5) * 5   -- 5 degree between pitch lines
  lineCount   = 16                             -- +/- 85 degree  
  centerX     = math.floor(x+w/2)
  centerY     = math.floor(y+h/2)
  minY        = y
  maxY        = y + h
  minX        = x
  maxX        = x + w
  yy          = 0  
  winglen     = w / 8  
 
  lcd.font(FONT_STD)
  STD_w, STD_h = lcd.getTextSize("")   
  lcd.font(FONT_XL)
  XL_w, XL_h = lcd.getTextSize("330")   
  lcd.font(FONT_XXL)
  XXL_w, XXL_h = lcd.getTextSize("-180 ")        
  
end
-- #################################################################### 
-- #  drawLib.CheckEnvironment                                        #
-- #    Read environment varibles                                     #
-- #################################################################### 
function drawLib.CheckEnvironment (widget)
  local wl, hl = lcd.getWindowSize()
  --print ("### function CheckEnvironment()")    
  if widget.zoneWidth ~= wl or widget.zoneHeight ~= hl then
    widget.zoneWidth  = wl
    widget.zoneHeight = hl
    
    local version = system.getVersion()
    widget.screenHeight = version.lcdHeight
    widget.screenWidth  = version.lcdWidth
    conf.simulation     = version.simulation
    conf.transmitter    = libs.utils.checkTransmitter(version.board)
    conf.modelName      = model.name()
    conf.darkMode       = lcd.darkMode() 	
        
    drawLib.changeDrawVar (widget)
    --print("#### CheckEnvironment zone size : " .. widget.zoneWidth .. "x" .. widget.zoneHeight)
  end

end
-- #################################################################### 
-- #  drawLib.readSensors                                              #
-- #################################################################### 
function drawLib.readSensors(widget)
  if conf.telemetryState == 1 then
    --widget.idRoll  = system.getSource({category=CATEGORY_TELEMETRY_SENSOR, appId=0x0440})     -- iNAV   
    --widget.idPitch = system.getSource({category=CATEGORY_TELEMETRY_SENSOR, appId=0x0430})     -- iNAV
    widget.roll   = drawLib.getSourceValue(widget.idRoll)
    widget.pitch  = drawLib.getSourceValue(widget.idPitch)
    widget.vAlt   = drawLib.getSourceValue(widget.idAlt)
    widget.vSpeed = drawLib.getSourceValue(widget.idVertSpeed)
    
--  widget.pitch  = -10    
--  widget.roll   = 20
--  widget.vAlt   = 120
--  widget.vSpeed = 2.2

    if widget.idAlt ~= nil then
      widget.hasVario = true
    end
  end
end
-- #################################################################### 
-- #  paintHorizon                                                    #
-- #################################################################### 
function drawLib.paintHorizon (widget)
  -- ********************************************************
  -- * drawLineWithClipping       paintHorizon() local  *
  -- ********************************************************
  local function drawLineWithClipping(x1,y1,x2,y2,style,xmin,xmax,ymin,ymax,color)
    local x = {}
    local y = {}
    if not(x1 < xmin and x2 < xmin) and not(x1 > xmax and x2 > xmax) then
      if not(y1 < ymin and y2 < ymin) and not(y1 > ymax and y2 > ymax) then
        x[1]=x1
        y[1]=y1
        x[2]=x2
        y[2]=y2
        for i=1,2
        do
          if x[i] < xmin then
            x[i] = xmin
            y[i] = ((y2-y1)/(x2-x1))*(xmin-x1)+y1
          elseif x[i] > xmax then
            x[i] = xmax
            y[i] = ((y2-y1)/(x2-x1))*(xmax-x1)+y1
          end

          if y[i] < ymin then
            y[i] = ymin
            x[i] = ((x2-x1)/(y2-y1))*(ymin-y1)+x1
          elseif y[i] > ymax then
            y[i] = ymax
            x[i] = ((x2-x1)/(y2-y1))*(ymax-y1)+x1
          end
        end
        if not(x[1] < xmin and x[2] < xmin) and not(x[1] > xmax and x[2] > xmax) then
          lcd.color(color)
          lcd.pen(style)
          lcd.drawLine(x[1],y[1],x[2],y[2])
        end
      end
    end
  end  
  -- ********************************************************
  -- * drawLineByOriginAndAngle   paintHorizon() local  *
  -- ********************************************************
  local function drawLineByOriginAndAngle(ox,oy,angle,len,style,xmin,xmax,ymin,ymax,color)
    -- faster then math library else "CPU limit"
    local xx = libs.utils.cos(angle) * len * 0.5
    local yy = libs.utils.sin(angle) * len * 0.5
    
    local x0 = ox - xx
    local x1 = ox + xx
    local y0 = oy - yy
    local y1 = oy + yy

    --tohle ne-- drawLineWithClipping (ox, oy, x1, y1, style, xmin, xmax, ymin, ymax, color)
    drawLineWithClipping(x0,y0,x1,y1,style,xmin,xmax,ymin,ymax,color)  
  end  
  -- --------------------------------------------------------------------
  -- drawLib.paintHorizon() code starts here ----------------------------
  -- --------------------------------------------------------------------
  
  local cx, cy, dx, dy
  local r = -widget.roll
  -- no roll ==> segments are vertical, offsets are multiples of R2
  if widget.roll == 0 or math.abs(widget.roll) == 180 then
    dx = 0
    dy = widget.pitch * scalePitch
    cx = 0
    cy = lineOffset
  else
    dx = libs.utils.cos(90 - r) *    widget.pitch * scalePitch
    dy = libs.utils.sin(90 - r) *  - widget.pitch * scalePitch
    -- 1st line offsets
    cx = libs.utils.cos(90 - r) * lineOffset
    cy = libs.utils.sin(90 - r) * lineOffset
  end
 
  local ox = x + w/2 + dx
  local oy = y + h/2 + dy
  
  -- Erase the area using sky color
  lcd.color(widget.colorSky)
  lcd.drawFilledRectangle(x, y, w, h) 
    
  -- ##################################
  -- ##   horizon land area          ##
  -- ##################################  
  lcd.color(widget.colorLand)  
  local angle = math.tan(math.rad(-widget.roll))
  -- Fill of the "land" side
  -- prevent divide by zero
  if widget.roll == 0 then
    --print("#### ".. minX .. " " .. minY .. " " .. maxX .. " " .. maxY .. " " .. dy)
    lcd.drawFilledRectangle(minX,      math.max(minY,dy+minY+(maxY-minY)/2),
                            maxX-minX, math.max(0,math.min(maxY-minY,(maxY-minY)/2-dy+(math.abs(dy) > 0 and 1 or 0))))
  elseif math.abs(widget.roll) >= 180 then
    lcd.drawFilledRectangle(minX,minY,maxX-minX,math.min(maxY-minY,(maxY-minY)/2+dy))
  else  -- true if flying inverted 
    local inverted = math.abs(widget.roll) > 90
    -- true if part of the hud can be filled in one pass with a rectangle
    local fillNeeded = false
    local yRect = inverted and 0 or h --LCD_H

    local step = 2
    local steps = (maxY - minY)/step - 1
    local yy = 0

    if 0 < widget.roll and widget.roll < 180 then
      for s=0,steps do
        yy = minY + s*step
        xx = ox + (yy-oy)/angle
        if xx >= minX and xx <= maxX then
          lcd.drawFilledRectangle(xx, yy, maxX-xx+1, step)
        elseif xx < minX then
          yRect = inverted and math.max(yy,yRect)+step or math.min(yy,yRect)
          fillNeeded = true
        end
      end
    elseif -180 < widget.roll and widget.roll < 0 then
      for s=0,steps do
        yy = minY + s*step
        xx = ox + (yy-oy)/angle
        if xx >= minX and xx <= maxX then
          lcd.drawFilledRectangle(minX, yy, xx-minX, step)
        elseif xx > maxX then
          yRect = inverted and math.max(yy,yRect)+step or math.min(yy,yRect)
          fillNeeded = true
        end
      end
    end

    if fillNeeded then
      local yMin = inverted and minY or yRect
      local height = inverted and yRect - minY or maxY-yRect
      lcd.drawFilledRectangle(minX, yMin, maxX-minX, height)
    end
    
  end
  -- central horizon line
  lcd.color(COLOR_RED)
  lcd.drawLine(x, centerY, x+w, centerY, 0x77)
  
  -- ##################################
  -- ##   pitch scale                ##
  -- ##################################
  local dLen
  local scale_dx = winglen / 2
  local lineEven = winglen / 2 + scale_dx
  for dist = 1, lineCount do
   if math.fmod (dist, 2) == 0 then
     dLen = lineEven
     lineEven = lineEven + scale_dx
   else
     dLen = winglen / 2
   end
    drawLineByOriginAndAngle(centerX + dx - dist*cx, oy + dist*cy, r, dLen, DOTTED, minX+frameX, maxX-frameX, minY+frameY, maxY-frameY, COLOR_WHITE)
    drawLineByOriginAndAngle(centerX + dx + dist*cx, oy - dist*cy, r, dLen, DOTTED, minX+frameX, maxX-frameX, minY+frameY, maxY-frameY, COLOR_WHITE)
  end  
  -- ##################################
  -- ##   roll scale                 ##
  -- ##################################
  -- roll marker offsets
  local rollMarkerArr      = {0, 10, 20, 30, 45, 60}
  local rollMarkerWidtwArr = {1,  0,  0,  1,  0,  1}
  local rollScale = math.floor((w - (2 * frameX)) / 150)    -- 150 deg = +/- 60deg + 2 * 15 def space
  for sign = -1, 1, 2 do
    for idx = 1, #rollMarkerArr do
      if rollMarkerWidtwArr[idx] == 1 then
        fromX = -1
        toX   =  1
      else
        fromX = 0
        toX   = 0
      end
        for mlcnt = fromX, toX, 1 do
          lcd.pen(SOLID)
          lcd.color(COLOR_WHITE)
          lcd.drawLine(centerX - sign * rollScale * rollMarkerArr[idx] + mlcnt, y, centerX - sign * rollScale * rollMarkerArr[idx] + mlcnt, y + frameY / 2)
        end
    end
  end
  
  -- ##################################
  -- ##   roll marker                ##
  -- ##################################
  local rollrange
  if widget.roll > 90 then             -- correct inverse flight
    rollrange = widget.roll - 180
  elseif widget.roll < -90 then
    rollrange = widget.roll + 180
  else
    rollrange = widget.roll
  end
  
  if rollrange < 65 and rollrange > -65 then
    lcd.drawBitmap(centerX - 7 +  rollrange * rollScale, y + frameY / 2, widget.upArrow)
  end
    
  --------------------------------
  --       V A L U E S          --
  --------------------------------
  -- roll value bottom center MIDSIZE
  lcd.color(COLOR_BLACK)
  lcd.font(FONT_XXL)
  lcd.drawFilledRectangle(centerX - XXL_w / 2, maxY-XXL_h, XXL_w, XXL_h)
  lcd.color(COLOR_WHITE)

  lcd.drawNumber(centerX, maxY-XXL_h + 5,  widget.roll, 0, 0, TEXT_CENTERED)
  
  if widget.hasVario == true then
    ------------------------------------------
    -- left part Altitude from vario        --
    ------------------------------------------
    -- gray background    
    -- 30% opacity of BLACK
    lcd.color(lcd.RGB(0,0,0,0.3))
    lcd.drawFilledRectangle(x, y, frameX, h)    
   -- lcd.drawBitmap(drawLib.getOpacityImg(h),x,y)
    -- vario altitude scale
    lcd.color(COLOR_WHITE)
    lcd.drawLine(x,y,x,y+h-1)
    
    -- markers
    local marker_len = 4
    local scaleAlt = math.floor(h / 60)
    local AltZero = centerY + widget.vAlt * scaleAlt   -- pocet pixlu kde zacina nula
    lcd.font(FONT_STD)
    for dist=0,199 do
      markerY = AltZero - 5 * dist * scaleAlt
      if (markerY < (y + frameY / 2)) then
        break
      elseif (markerY < (y + h- frameY/2)) then
        lcd.drawLine(x, markerY, x+marker_len, markerY)
        if math.fmod (dist, 2) == 0 then
          lcd.drawNumber(x+(frameX - marker_len) / 2 + marker_len, markerY-9, 5 * dist, 0, 0, TEXT_CENTERED)
        end
      end
    end
    lcd.font(FONT_XL)
    lcd.color(COLOR_BLACK)
    lcd.drawFilledRectangle(x, centerY-XL_h/2, XL_w, XL_h)
    lcd.color(COLOR_WHITE)
    lcd.drawRectangle(x, centerY-XL_h/2, XL_w, XL_h)
    -- vario Altitude - left frame
    lcd.drawNumber(x+XL_w/2, centerY - XL_h/2 + 2, widget.vAlt, 0, 0, TEXT_CENTERED)
    -- Altitude unit
    lcd.font(FONT_STD)    
    lcd.drawText(x+frameX + 4, y + frameY, "Alt m")
    
    ------------------------------------------
    -- right part Vertical Speed from vario --
    ------------------------------------------
    -- gray background    
    -- 30% opacity of BLACK
    lcd.color(lcd.RGB(0,0,0,0.3))
    lcd.drawFilledRectangle(w - frameX, y, frameX, h)   
    -- vario Vertical Speed scale
    lcd.color(COLOR_WHITE)
    lcd.drawLine(x+w-1,y,x+w-1,y+h-1) 
    -- markers
    local scalevSpd = 8
    local vSpdZero = centerY + 5 * widget.vSpeed * scalevSpd   -- pocet pixlu kde zacina nula
    lcd.font(FONT_STD)    
    for sign = -1, 1, 2 do
      for dist=0,50 do
        markerY = vSpdZero - dist * scalevSpd * sign
        if (markerY >= (y + frameY/2)) and (markerY < (y + h- frameY/2)) then      
          lcd.drawLine(x+w-marker_len-1, markerY, x+w-1, markerY)
          if math.fmod (dist, 5) == 0 then
            lcd.drawLine(x+w-marker_len-4, markerY, x+w-1, markerY)
            lcd.drawNumber(x+w-(frameX-marker_len) /2 - marker_len, markerY-9, sign * dist / 5, 0, 0, TEXT_CENTERED)
          end        
        end
      end
    end
    ------------------------------------------
    -- Vertical Speed value                 --
    ------------------------------------------
    lcd.font(FONT_XL)
    lcd.color(widget.vSpeed > 1 and COLOR_GREEN or (widget.vSpeed < -1 and COLOR_RED or COLOR_BLACK))
    lcd.drawFilledRectangle(x+w-XL_w, centerY-XL_h/2, XL_w, XL_h)
    lcd.color(COLOR_WHITE)
    lcd.drawRectangle(x+w-XL_w, centerY-XL_h/2, XL_w, XL_h)
    
    -- vario vertical speed - right frame
    lcd.color(widget.vSpeed > 1 and COLOR_BLACK or COLOR_WHITE)    
    lcd.drawNumber(x + w - XL_w / 2, centerY - XL_h/2 + 2, widget.vSpeed, 0, 1, TEXT_CENTERED)
    lcd.font(FONT_STD)
    lcd.color(COLOR_WHITE)
    -- Vertical speed unit
    lcd.drawText(x+w-frameX, y + frameY, "m/s", TEXT_RIGHT)    
  end   -- has vario
  ------------------------------------------
  -- pitch value                          --
  ------------------------------------------
  -- 50% opacity of BLACK
  lcd.color(lcd.RGB(0,0,0,0.5))
  lcd.font(FONT_XL)
  lcd.drawFilledRectangle(centerX - 3 * winglen / 2 - XL_w, centerY - XL_h / 2, XL_w, XL_h)
  lcd.color(COLOR_WHITE)  
  lcd.drawNumber(centerX - 3 * winglen / 2 - XL_w /2,centerY - XL_h / 2, widget.pitch, 0, 0, TEXT_CENTERED)
  
    -- center rectangle
  lcd.color(COLOR_YELLOW)
  lcd.drawFilledRectangle(centerX - 4, centerY - 4, 8, 8)
  lcd.color(COLOR_RED)
  lcd.drawFilledRectangle(centerX - 2, centerY - 2, 4, 4)
  lcd.color(COLOR_YELLOW)
  -- aircraft symbol
  -- L wing
  lcd.drawFilledRectangle(centerX - winglen - winglen / 2, centerY - 2, winglen, 4)
  lcd.drawFilledRectangle(centerX - winglen / 2 - 4, centerY + 2, 4, 4)
  -- R wing
  lcd.drawFilledRectangle(centerX + winglen / 2 , centerY - 2, winglen, 4)
  lcd.drawFilledRectangle(centerX + winglen / 2 ,  centerY + 2, 4, 4)  
end
-- #################################################################### 
-- # drawLib.paint                                                     #
-- ####################################################################
function drawLib.paint (widget)
  libs.drawLib.CheckEnvironment (widget)                            -- zone change       
  libs.drawLib.readSensors(widget)

  if conf.transmitter ~= 20 then
    libs.utils.printError (widget, "nohwsup")
  else
    drawLib.paintHorizon(widget)  
    
    if conf.telemetryState == 0 then
      lcd.color(COLOR_RED)
      lcd.drawRectangle(0, 0, widget.zoneWidth, widget.zoneHeight, widget.noTelFrameT)  
    end
  end
end

return drawLib