-- #########################################################################
-- #                                                                       #
-- # License GPLv3: https://www.gnu.org/licenses/gpl-3.0.html              #
-- #                                                                       #
-- # This program is free software; you can redistribute it and/or modify  #
-- # it under the terms of the GNU General Public License version 2 as     #
-- # published by the Free Software Foundation.                            #
-- #                                                                       #
-- # This program is distributed in the hope that it will be useful        #
-- # but WITHOUT ANY WARRANTY; without even the implied warranty of        #
-- # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
-- # GNU General Public License for more details.                          #
-- #                                                                       #
-- #########################################################################
--
-- =============================================================================
-- ETHOS artificial horizon
-- File:   : main.lua
-- Author  : © RNDr.Vladimir Pribyl, CSc. (VPRHELI)
-- History : Date        Version Author   Comment
--           ----------  ------- -------- ------------------------------------
--           16.02.2025  0.0.1   VPRHELI  initial version
-- =============================================================================
--
-- TODO

local version           = "v1.0.0"
local environment       = system.getVersion()
-- load translate table from external file
local tableFile  = assert(loadfile("/scripts/arthor/translate.lua"))()
local transtable = tableFile.transtable
local utils      = {}
local libs       = { menuLib  = nil,
                     drawLib  = nil,
                     utils    = nil}
local g_libInitDone = false

-- #############
-- # conf      #
-- #############
 local conf = {
                version        = version,
                locale         = system.getLocale(),
                basePath       = "/scripts/arthor/",
                libFolder      = "lib/",
                imgFolder      = "img/",
                modelName      = nil,
                transmitter    = nil,
                transtable     = transtable,
                telemetryState = nil,
                lastTelState   = nil,       -- last telemetry state
                simulation     = nil,
                darkMode       = nil,
              }

local g_rescan_seconds  = 5   -- check how often rescan ID (find new LS ....)
local g_paint_frequency = 4   -- how many times per second display will be updated (4x per second)

-- #################################################################### 
-- # loadLibrary                                                      #
-- ####################################################################
function loadLibrary(filename)
  local lib = dofile(conf.basePath .. conf.libFolder .. filename..".lua")
  if lib.init ~= nil then
    lib.init(conf, libs)
  end
  return lib
end
-- #################################################################### 
-- *  name                                                            #
-- *    return Widget name                                            #
-- #################################################################### 
local function name(widget)
  return libs.utils.translate ("wgname")
end
-- #################################################################### 
-- *  initLibraries                                                   #
-- *    return Widget name                                            #
-- #################################################################### 
local function initLibraries(widget)
  --print ("### initLibraries()")
  if g_libInitDone == false then
    g_libInitDone = true
    -- load libraries
    libs.utils   = loadLibrary("utils")
    libs.menuLib = loadLibrary("menuLib")
    libs.drawLib = loadLibrary("drawLib")
  end   
end
-- #################################################################### 
-- #  create                                                          #
-- #    this function is called whenever the widget is first          #
-- #    initialised.                                                  #
-- #    it's usefull for setting up widget variables or just          #
-- #################################################################### 
local function create()
  --print ("### function create()")
  
	return { 
          -- telemetry
          -- ROLLPITCH
          idRoll          = nil,
          idPitch         = nil,
          roll            = 0,
          pitch           = 0,
          -- VARIO
          idAlt           = nil,
          idVertSpeed     = nil,
          vAlt            = 0,
          vSpeed          = 0,
          hasVario        = false,
          -- config
          colorLand      = lcd.RGB(224, 112,   0),   
          colorSky       = lcd.RGB(  0, 160, 224),   
          transtable     = transtable,
          -- status
          initPending    = true,
          runBgTasks     = false,
          FlightReset    = 0,          -- should be zero
          -- layout
          screenHeight   = nil,
          screenWidth    = nil, 
          zoneHeight     = 0,
          zoneWidth      = 0,
          screenType     = "",
          upArrow        = nil,
          flash          = 0,        -- some values can flash
          last_rescan    = 0,
          last_paint     = 0,
          noTelFrameT    = 3,        -- thickness of no telemetry frame
        }
end
-- #################################################################### 
-- *  paint                                                           #
-- ####################################################################
local function paint(widget)
  --print ("### function paint()")
  libs.drawLib.paint (widget)
end
-- #################################################################### 
-- # menu                                                             #
-- #    add a menu item to the configuration menu popup of the widget #
-- 3    usefull if adding new tools                                   #
-- ####################################################################
local function menu(widget)
  --print ("### function menu()")
	-- add a menu item to the configuration menu popup of the widget
	-- usefull if adding new tools

	return {
		--   { "Entry 1", function() end},
		--   { "Entry 2", function() end},
	}
end
-- #################################################################### 
-- # configure                                                        #
-- #    Widget Configuration options                                  #
-- #################################################################### 
local function configure(widget)
  --print ("### function configure()")
  libs.menuLib.configure (widget)
end
-- #################################################################### 
-- # read                                                             #
-- #    read values from internal storage                             #
-- #################################################################### 
local function read(widget)
  --print ("### function read()")
  widget.idRoll      = storage.read("senRoll")  
  widget.idPitch     = storage.read("senPitch")
  widget.idAlt       = storage.read("senAlt")  
  widget.idVertSpeed = storage.read("senvSpeed")  
  widget.colorLand   = storage.read("colorLand")  
  widget.colorSky    = storage.read("colorSky")
	return true
end
-- #################################################################### 
-- # write                                                            #
-- #    write values to internal storage                              #
-- #################################################################### 
local function write(widget)
  --print ("### function write()")
  storage.write("senRoll",    widget.idRoll)  
  storage.write("senPitch",   widget.idPitch)  
  storage.write("senAlt",     widget.idAlt)  
  storage.write("senvSpeed",  widget.idVertSpeed)  
  storage.write("colorLand",  widget.colorLand)  
  storage.write("colorSky",   widget.colorSky)  
	return true
end
-- #################################################################### 
-- # event                                                            #
-- #    trigger whenever the widget is in focus and and               #
-- #    even occurs such as a button or screen click                  #
-- #################################################################### 
local function event(widget, category, value, x, y)
  --print ("### function event()")
	--print ("### Event received:", category, value, x, y)
	
	return true
end
-- #################################################################### 
-- # wakeup                                                           #
-- #    this is the main loop that ethos calls every couple of ms     #
-- #################################################################### 
local function wakeup(widget)
  local actual_time = os.clock()  -- Získání aktuálního času
  --print ("### function wakeup()")
  if widget.initPending == true then
      -- TODO if necesssary
    libs.utils.checkTelemetry()
    conf.lastTelState  = conf.telemetryState
    widget.upArrow     = libs.utils.loadBitmap("upArrow.png")    
    widget.runBgTasks  = true
    widget.initPending = false
  end  
  
  if widget.runBgTasks == true then
    if lcd.isVisible() then
      if actual_time > widget.last_rescan then                            -- rescan environment, telemetry status
        widget.last_rescan = actual_time + g_rescan_seconds               -- new time for rescan
        widget.zoneWidth = 0
        libs.drawLib.CheckEnvironment (widget)                            -- zone change
        libs.utils.checkTelemetry()
        libs.utils.checkFlightReset(widget)
        if conf.telemetryState ~= conf.lastTelState then
          -- telemetry state changed
          --print ("### telemetry state changed")
          conf.lastTelState = conf.telemetryState
          -- let transmitter and widgets until sensors are back on
          widget.last_paint = actual_time  + 3
        end        
      end
      if actual_time > widget.last_paint then                             -- rescan environment, telemetry status
        widget.last_paint = actual_time + 1 / g_paint_frequency  -- new time for paint
        lcd.invalidate ()        
      end
    end     -- isVisible
  end       -- runBgTasks
  return
end

-- #################################################################### 
-- # init                                                             #
-- #    this is where we 'setup' the widget                           #
-- #################################################################### 
local function init()
  --print ("### function init()")
	local key = "arthor"			  -- unique key - keep it less that 8 chars

  initLibraries ()   -- load all becessary libraries

  system.registerWidget(
        {
            key       = key,				  -- unique project id
            name      = name,				  -- name of widget - objevi se v seznamu Widgetu
            create    = create,			  -- function called when creating widget
            configure = configure,		-- function called when configuring the widget (use ethos forms)
            paint     = paint,				-- function called when lcd.invalidate() is called
            wakeup    = wakeup,			  -- function called as the main loop
            read      = read,				  -- function called when starting widget and reading configuration params
            write     = write,				-- function called when saving values / changing values in the configuration menu
            event     = event,				-- function called when buttons or screen clips occur
            menu      = menu,				  -- function called to add items to the menu
            persistent = false,			  -- true or false to make the widget carry values between sessions and models (not safe imho)
        }
  )
  
end

return {init = init}
